<?php

//
// HORIZONT Software GmbH, Munich
//



    require_once('wai_functions.php');
    require_once('hwm_lib.php');



    function snow_pre_action()
    {
        if (!isset($_SESSION['hwm_rest_api_data']['SNOW_ID']))
            return true;
        $action = hwm_get_action();
        if ($action === false)
            return false;
        if (!isset(SNOW_TICKET_CONFIRM_ACTIONS[$action]))
            return true;
        $stage = SNOW_TICKET_CONFIRM_ACTIONS[$action];
        $process = hwm_get_process();
        if ($process === false)
            return false;
        $value = $_SESSION['hwm_rest_api_data']['SNOW_ID'];
        if (!snow_set_process_param($process, 'SNOW_ID_'.$stage, $value))
            return false;
        return true;
    }



    function snow_request_private(&$connection, $data, &$errMsg)
    {
        if (!isset($data['status'])
            || !isset($data['activity']['step'])
            || !isset($data['activity']['process'])
            || !isset($data['activity']['action']))
            return 0; // abort - do not contact SNOW
        $process = $data['activity']['process'];
        $action = $data['activity']['action'];
        if (isset(SNOW_TICKET_OPEN_ACTIONS[$action]))
        {
            $type = 'OPEN';
            $stage = SNOW_TICKET_OPEN_ACTIONS[$action];
        }
        else if (isset(SNOW_TICKET_CONFIRM_ACTIONS[$action]))
        {
            $type = 'CONFIRM';
            $stage = SNOW_TICKET_CONFIRM_ACTIONS[$action];
        }
        else if (isset(SNOW_TICKET_CLOSE_ACTIONS[$action]))
        {
            $type = 'CLOSE';
            $stage = SNOW_TICKET_CLOSE_ACTIONS[$action];
        }
        else
            return 0; // abort - do not contact SNOW
        $status = $data['status'];
        if (($status !== 'Complete' || $type === 'CONFIRM')
            && ($status !== 'Rejected' || $type === 'OPEN'))
            return 0; // abort - do not contact SNOW
        if ($type === 'CONFIRM')
            $type = 'CLOSE';
        $request = array();
        if ($type === 'OPEN')
        {
            $apprReq = !isset($data['RITM_SYS_ID']);
            snow_context_data(array(
                'instruction' => $apprReq ? 'APPR_REQ' : 'UPDATE',
                'process' => $process,
                'param' => 'SNOW_ID_'.$stage,
                'context' => $data,
            ));
            if ($apprReq)
            {
                $url = SNOW_URL_APPR_REQ;
                $memo = snow_get_process_memo($process);
                if ($memo === false)
                {
                    $errMsg = hwm_get_last_error();
                    return false;
                }
                $memo = str_replace(array("\r\n", "\n"), array("\n", "\r\n"),
                                    $memo);
                $app = isset($data['additional']['BUSINESS_APP'])
                       ? $data['additional']['BUSINESS_APP'] : '';
                $app = snow_map_app_to_id($app);
                $proj = isset($data['additional']['PROJECT_ID'])
                        ? $data['additional']['PROJECT_ID'] : '';
                $date = isset($data['additional'][$stage.'_DATE'])
                        ? $data['additional'][$stage.'_DATE'] : '';
                $date = hwm_db_date_adjust($date);
                if ($date === false)
                    $date = '';
                else if (strlen($date) === 8)
                    $date = substr($date, 6, 2).'-'.substr($date, 4, 2).'-'
                            .substr($date, 0, 4);
                $descr = isset($data['general']['process_description'])
                         ? $data['general']['process_description'] : '';
                $request['sysparm_quantity'] = '1';
                $request['variables'] = array(
                    'environment' => $stage,
                    'application' => $app,
                    'release_date' => $date,
                    'description' => $memo,
                    'project' => $proj,
                    'title' => $descr,
                    'activity_id' => (string) ($data['activity']['step'] + 1),
                );
                $request['sysparm_item_guid'] = SNOW_SYS_ID;
                $request['get_portal_messages'] = 'true';
                $request['sysparm_no_validation'] = 'true';
                $request['engagement_channel'] = 'sp';
                $request['referrer'] = null;
            }
            else
            {
                // Currently not used as the activity_id is sent in the first
                // request so no need to send it as correlation_id here.
                $url = str_replace('<sys_id>', $data['RITM_SYS_ID'],
                                   SNOW_URL_UPDATE);
                $pmID = $data['activity']['step'] + 1;
                $msg = str_replace('<procman_id>', $pmID, SNOW_UPDATE_MSG);
                $request['work_notes'] = $msg;
                $request['correlation_id'] = (string) $pmID;
            }
            $connection['url'] .= $url;
        }
        else
        {
            snow_context_data(array(
                'instruction' => 'CLOSE',
            ));
            $param = 'SNOW_ID_'.$stage;
            if (!isset($data['additional'][$param]))
            {
                $errMsg = 'Missing SNOW ticket in the parameter \''.$param
                          .'\' of the process '.$process.'!';
                return false;
            }
            $ticket = $data['additional'][$param];
            $connection['url'] .= str_replace('<sys_id>', $ticket,
                                              SNOW_URL_CLOSE);
            $connection['customrequest'] = 'PUT';
            $request['work_notes'] = SNOW_CLOSE_MSG;
            $request['state'] = $status === 'Complete' ? '3' : '7';
        }

        $connection['httpheader'] = array(
            'Content-Type: application/json;charset=UTF-8',
            'Authorization: Basic '.SNOW_AUTH,
        );
        $connection['postfields'] = json_encode($request);
        snow_log('request', $connection['postfields'], $connection['url']);
        return true;
    }



    function snow_response_private(&$response, &$errMsg)
    {
        snow_log('response', $response);
        $data = json_decode($response, true);
        if (isset($data['result']))
            $data = $data['result'];
        $context = snow_context_data();
        $instruction = isset($context['instruction'])
                       ? $context['instruction'] : '';
        if (!is_string($instruction))
            $instruction = '';
        switch ($instruction)
        {
        case 'APPR_REQ':
            if (!isset($data['sys_id'])
                || !is_string($data['sys_id'])
                || !preg_match('/^[0-9a-f]{32}$/', $data['sys_id']))
            {
                $errMsg = 'Missing or invalid member "sys_id" in the SNOW '
                          .'approval response!';
                return false;
            }
            if (!isset($data['number'])
                || !is_string($data['number'])
                || !preg_match('/^REQ[0-9]{7}$/', $data['number']))
            {
                $errMsg = 'Missing or invalid member "number" in the SNOW '
                          .'response!';
                return false;
            }
            if (!isset($data['request_number'])
                || $data['request_number'] !== $data['number'])
            {
                $errMsg = 'Missing or invalid member "request_number" in the '
                          .'SNOW response!';
                return false;
            }
            $process = $context['process'];
            $param = $context['param'];
            if (!snow_set_process_param($process, $param, $data['number']))
            {
                $errMsg = hwm_get_last_error();
                return false;
            }
            /*
            // Currently not used as the activity_id is sent in the first
            // request so no need to send it as correlation_id here.
            $context = $context['context'];
            $context['RITM_SYS_ID'] = $data['sys_id'];
            $result = hwm_universal_interface_exec('snow', $context);
            if ($result === false)
            {
                $errMsg = hwm_get_last_error();
                return false;
            }
             */
            break;
        case 'UPDATE':
            if (!isset($data['number'])
                || !is_string($data['number'])
                || !preg_match('/^RITM[0-9]{7}$/', $data['number']))
            {
                $errMsg = 'Missing or invalid member "number" in the SNOW '
                          .'update response!';
                return false;
            }
            $process = $context['process'];
            $param = $context['param'];
            if (!snow_set_process_param($process, $param, $data['number']))
            {
                $errMsg = hwm_get_last_error();
                return false;
            }
            break;
        case 'CLOSE':
            if (!isset($data['state'])
                || ($data['state'] !== '3' && $data['state'] !== '7'))
            {
                $errMsg = 'Missing or invalid member "state" in the SNOW '
                          .'update response!';
                return false;
            }
            break;
        default:
            $errMsg = 'Invalid instruction "'.$instruction.'"!';
            return false;
            break;
        }
        return true;
    }



    function snow_log($type, $jsonData, $url = false)
    {
        if (!SNOW_LOG)
            return;
        switch ($type)
        {
        case 'request':
            $msg = 'Sending request to SNOW:';
            if (is_string($url))
                $msg .= PHP_EOL.'URL: '.$url;
            break;
        case 'response':
            $msg = 'Received response from SNOW:';
            break;
        default:
            return;
            break;
        }
        error_log(sprintf('[%s] %s', date('Y-m-d H:i:s'), $msg).PHP_EOL, 3,
                  SNOW_LOG_FILE);
        $jsonData = json_decode($jsonData, true);
        $jsonData = json_encode($jsonData, JSON_PRETTY_PRINT);
        $jsonData = str_replace("\n", PHP_EOL, $jsonData);
        error_log($jsonData.PHP_EOL, 3, SNOW_LOG_FILE);
    }



    function snow_context_data($data = null)
    {
        static $contextData = array();
        if (isset($data))
            $contextData = $data;
        return $contextData;
    }



    function snow_get_process_memo($process)
    {
        if (!hwm_db_lock(array('client' => false, 'process_memo' => false)))
            return false;
        $memos = hwm_get_process_memo($process);
        if ($memos === false)
        {
            hwm_db_rollback();
            return false;
        }
        hwm_db_rollback();
        if (!is_array($memos))
            return '';
        $memo = '';
        foreach ($memos as $item)
        {
            $ts = hwm_db_ts_adjust($item['timestamp']);
            if ($ts === false)
                $ts = '(no timestamp)';
            else if (strlen($ts) === 14)
                $ts = substr($ts, 0, 4).'-'.substr($ts, 4, 2).'-'
                      .substr($ts, 6, 2).' '.substr($ts, 8, 2).':'
                      .substr($ts, 10, 2).':'.substr($ts, 12, 2);
            $text = $ts.' '.$item['user'].':'."\n".$item['text']."\n";
            $text = str_replace(array("\r\n", "\n"), array("\n", "\r\n"),
                                $text);
            if ($memo !== '')
                $text .= "\n";
            $memo = $text.$memo;
        }
        return $memo;
    }



    function snow_map_app_to_id($app)
    {
        if (!is_string($app))
            return '';
        $id = hwm_expression_ext_db_table_map(
                  SNOW_CUSTOM_DATA_APP_TYPE, SNOW_CUSTOM_DATA_ID_COLUMN,
                  SNOW_CUSTOM_DATA_APP_COLUMN, $app);
        hwm_db_rollback();
        if (!is_string($id))
            $id = '';
        return $id;
    }



    function snow_set_process_param($process, $param, $val)
    {
        if (!hwm_db_lock(array('client' => false, 'param' => true)))
            return false;
        if (!hwm_set_process_param($process, $param, $val))
        {
            hwm_db_rollback();
            return false;
        }
        /*
         * The following code requires ProcMan 5.0.10! Then it can replace the
         * raw commit below.
        if (!hwm_db_commit())
            return false;
         */
        global $hwm_db_connection;
        hwm_log(HWM_LOG_TRC, 'COMMIT: LOG_ID='.hwm_log_id());
        if (db_commit($hwm_db_connection) === false)
        {
            hwm_db_error();
            return false;
        }
        return true;
    }
?>
